@@ -5,10 +5,24 @@
 #' entries are named, and will use the value as name when it is missing.
 #'
 #' @param ... dot arguments as passed to
+#' @param .character_only logical indicating whether \code{...} can be assumed
+#'   to be charater strings.
+#'
+#' @noRd
 #' @return A named character vector.
-symbol_list <- function(...)
+symbol_list <- function(..., .character_only = FALSE, .all=FALSE)
 {
-  dots    <- eval(substitute(alist(...)), parent.frame(), parent.frame())
+  if (isTRUE(.character_only)) {
+    dots <- unlist(list(...))
+  } else {
+    dots    <- eval(substitute(alist(...)), parent.frame(), parent.frame())
+  }
+
+  if (length(dots)==0) {
+    # If .all was true, empty dots should no longer error
+    return(character())
+  }
+
   names   <- names(dots)
   unnamed <- if (is.null(names)) 1:length(dots) else which(names == "")
   dots    <- vapply(dots, symbol_as_character, character(1))
