@@ -9,11 +9,11 @@ cleanup_environment <- function(environments=c("imports"),
                                 cleanup_here=FALSE, exclude_self=TRUE) {
   for (env_name in environments) {
     if(env_name %in% search())
-      rm(list=ls(n=env_name), pos=env_name)
+      rm(list=ls(n=env_name, all.names=TRUE), pos=env_name)
   }
   # Cleanup the local environment, if applicable
   if (cleanup_here) {
-    obj_list <- ls(pos=parent.env(environment()))
+    obj_list <- ls(pos=parent.env(environment()), all.names=TRUE)
     if (exclude_self)
       obj_list <- setdiff(obj_list,"cleanup_environment")
     rm(list=obj_list, pos=parent.env(environment()))
